//****************************************************************************
// File:    ropopup.c
//
// Purpose: This sample demonstrates a read-only popup window
//
// Functions:
//    WinMain()         - calls initialization function, processes message loop
//    InitApplication() - initializes window data and registers window
//    InitInstance()    - saves instance handle and creates main window
//    MainWndProc()     - processes messages
//    About()           - processes messages for "About" dialog box
//    InitROPopup()     - registers read-only popup window class
//    ROPopupProc()     - processes messages for read-only popup window
//
// Development Team:
//    Patrick Schreiber
//
// Written by Microsoft Product Support Services, Windows Developer Support
// Copyright (c) 1992 Microsoft Corporation. All rights reserved.
//****************************************************************************


#include <windows.h>     // required for all Windows applications
#include <string.h>      // required for strlen()
#include "ropopup.h"     // specific to this program


// Global variables
HANDLE ghInst;           // current instance
int    cxChar, cyChar;   // character width and height
int    xScreen, yScreen; // screen width and height


//****************************************************************************
// Function: WinMain()
//
// Purpose:  Calls initialization function, processes message loop
//
// Parameters:
//    HANDLE hInstance
//    HANDLE hPrevInstance
//    LPSTR  lpCmdLine
//    int    nCmdShow
//
// Returns:
//    int
//
// Comments:
//
// History:    Date        Author         Reason
//
//****************************************************************************

int PASCAL WinMain(HANDLE hInstance,     // current instance
                   HANDLE hPrevInstance, // previous instance
                   LPSTR  lpCmdLine,     // command line
                   int    nCmdShow)      // show-window type (open/icon)
{
   MSG msg; // message

   if (!hPrevInstance)                   // Other instances of app running?
      if (!InitApplication(hInstance))   // Initialize shared things
         return FALSE;                   // Exits if unable to initialize

   // Perform initializations that apply to a specific instance

   if (!InitInstance(hInstance, nCmdShow))
      return FALSE;

   // Acquire and dispatch messages until a WM_QUIT message is received

   while (GetMessage(&msg,     // message structure
                     NULL,     // handle of window receiving the message
                     NULL,     // lowest message to examine
                     NULL))    // highest message to examine
   {
      TranslateMessage(&msg);  // Translates virtual key codes
      DispatchMessage(&msg);   // Dispatches message to window
   }

   return msg.wParam;          // Returns the value from PostQuitMessage
}


//****************************************************************************
// Function: InitApplication()
//
// Purpose:  Initializes window data and registers window class
//
// Parameters:
//    HANDLE hInstance
//
// Returns:
//    BOOL
//
// Comments:
//
// History:    Date        Author         Reason
//
//****************************************************************************

BOOL InitApplication(HANDLE hInstance) // current instance
{
   WNDCLASS wc;                        // Window class structure
   char     szMenuName[STR_LEN];       // Menu name
   char     szClassName[STR_LEN];      // Class name

   // Load menu and class name from string table

   LoadString(hInstance, IDS_MENUNAME, szMenuName, STR_LEN);
   LoadString(hInstance, IDS_CLASSNAME, szClassName, STR_LEN);

   // Fill in the window class structure

   wc.style         = NULL;            // Class style(s)
   wc.lpfnWndProc   = MainWndProc;     // Class window procedure
   wc.cbClsExtra    = 0;               // No per-class extra data
   wc.cbWndExtra    = 0;               // No per-window extra data
   wc.hInstance     = hInstance;       // Application that owns the class
   wc.hIcon         = LoadIcon(hInstance, "appicon");
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = COLOR_WINDOW + 1;
   wc.lpszMenuName  = szMenuName;      // Name of menu resource in .RC file
   wc.lpszClassName = szClassName;     // Name used in call to CreateWindow

   // Register the window class

   if (RegisterClass(&wc))
      // Register the read-only popup class
      return InitROPopup(hInstance);
   else
      return FALSE;
}


//****************************************************************************
// Function: InitInstance()
//
// Purpose:  Saves instance handle and creates main window
//
// Parameters:
//    HANDLE hInstance
//    int    nCmdShow
//
// Returns:
//    BOOL
//
// Comments:
//
// History:    Date        Author         Reason
//
//****************************************************************************

BOOL InitInstance(HANDLE hInstance,  // Current instance identifier
                  int    nCmdShow)   // Param for first ShowWindow() call
{
   HWND hWnd;                        // Main window handle
   char szClassName[STR_LEN];        // Class name
   char szCaption[STR_LEN];          // Window caption

   // Load class name and window caption from string table

   LoadString(hInstance, IDS_CLASSNAME, szClassName, STR_LEN);
   LoadString(hInstance, IDS_CAPTION, szCaption, STR_LEN);

   // Save the instance handle in static variable, which will be used in
   // many subsequence calls from this application to Windows

   ghInst = hInstance;

   // Create a main window for this application instance

   hWnd = CreateWindow(szClassName,         // Window class name
                       szCaption,           // Window caption
                       WS_OVERLAPPEDWINDOW, // Window style
                       CW_USEDEFAULT,       // Default x-coord
                       CW_USEDEFAULT,       // Default y-coord
                       400,                 // Window width
                       300,                 // Window height
                       NULL,                // Parent or owner
                       NULL,                // Class menu
                       hInstance,           // Instance of owner
                       NULL);               // Pointer not needed

   // If window could not be created, return "failure"

   if (!hWnd)
      return FALSE;

   // Make the window visible and update its client area

   ShowWindow(hWnd, nCmdShow); // Show the window
   UpdateWindow(hWnd);         // Sends WM_PAINT message

   return TRUE;                // Returns the value from PostQuitMessage
}


//****************************************************************************
// Function: MainWndProc()
//
// Purpose:  Processes messages
//
// Parameters:
//    HWND   hWnd
//    UINT   message
//    WPARAM wParam
//    LPARAM lParam
//
// Returns:
//    long
//
// Comments:
//
// History:    Date        Author         Reason
//             7/7/92      Pat Schreiber  Created
//
//****************************************************************************

long FAR PASCAL MainWndProc(HWND   hWnd,    // window handle
                            UINT   message, // type of message
                            WPARAM wParam,  // additional information
                            LPARAM lParam)  // additional information
{
   FARPROC     lpProcAbout;        // Pointer to the "About" function
   static HWND hwndBtn, hwndPopup; // Handles to button and popup windows
   static int  btnX, btnY, btnWidth, btnHeight;

   switch (message)
   {
      case WM_CREATE:
         {
            HDC        hdc;
            TEXTMETRIC tm;
            RECT       rc;

            // Get screen dimensions and character ave. width and height

            hdc    = GetDC(hWnd);

            xScreen = GetDeviceCaps(hdc, HORZRES);
            yScreen = GetDeviceCaps(hdc, VERTRES);

            GetTextMetrics(hdc, &tm);
            cxChar = tm.tmAveCharWidth;
            cyChar = tm.tmHeight + tm.tmExternalLeading;

            ReleaseDC(hWnd, hdc);

            // Set button position to the center of the client area

            GetClientRect(hWnd, &rc);

            btnWidth  = (strlen("Press Me!") + 4) * cxChar;
            btnHeight = MulDiv(cyChar, 7, 4);
            btnX      = (rc.right - rc.left)/2 - btnWidth/2;
            btnY      = (rc.bottom - rc.top)/2 - btnHeight/2;

            // Create the pushbutton

            hwndBtn = CreateWindow("button",
                                   (LPSTR)"Press Me!",
                                   WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                                   btnX,
                                   btnY,
                                   btnWidth,
                                   btnHeight,
                                   hWnd,
                                   ID_BUTTON,
                                   ghInst,
                                   NULL);
         }
         break;

      case WM_SIZE:
         {
            RECT       rc;

            // Move the pushbutton to the center of the client area

            GetClientRect(hWnd, &rc);

            btnWidth  = (strlen("Press Me!") + 4) * cxChar;
            btnHeight = MulDiv(cyChar, 7, 4);
            btnX      = (rc.right - rc.left)/2 - btnWidth/2;
            btnY      = (rc.bottom - rc.top)/2 - btnHeight/2;

            MoveWindow(hwndBtn, btnX, btnY, btnWidth, btnHeight, TRUE);
         }
         break;

      case WM_COMMAND:  // message: command from application menu
         switch (wParam)
         {
            case IDM_ABOUT:
               {
                  lpProcAbout = MakeProcInstance(About, ghInst);

                  DialogBox(ghInst,       // current instance
                            "AboutBox",   // resource to use
                            hWnd,         // parent handle
                            lpProcAbout); // About() instance address

                  FreeProcInstance(lpProcAbout);
               }
               break;

            case ID_BUTTON:
               {
                  RECT  rc;
                  POINT pt;
                  int   popX, popY, popWidth, popHeight;

                  if (HIWORD(lParam) == BN_CLICKED)
                  {
                     GetClientRect((HWND)LOWORD(lParam), &rc);
                     pt.x = rc.left;
                     pt.y = rc.top;
                     ClientToScreen((HWND)LOWORD(lParam), &pt);

                     // Set the read-only popup position and dimensions

                     popWidth  = 500;
                     popHeight = 200;
                     popX      = pt.x - (popWidth/2 - btnWidth/2);
                     popY      = pt.y + btnHeight;

                     if (popX < 0)
                        popX = 0;
                     if (popY < 0)
                        popY = 0;
                     if ((popX + popWidth) > xScreen)
                        popX = xScreen - popWidth;
                     if ((popY + popHeight) > yScreen)
                        popY = yScreen - popHeight;

                     // Create the read-only popup

                     hwndPopup = CreateWindow("ropopup",
                                              NULL,
                                              WS_POPUP | WS_VISIBLE,
                                              popX,
                                              popY,
                                              popWidth,
                                              popHeight,
                                              hWnd,
                                              NULL,
                                              ghInst,
                                              NULL);
                  }
               }
               break;

            case IDM_EXIT:
               SendMessage(hWnd, WM_CLOSE, 0, 0L);
               break;

            default:    // Lets Windows process it
               return (DefWindowProc(hWnd, message, wParam, lParam));
         }
         break;

      case WM_CLOSE:
         DestroyWindow(hWnd);
         return 0L;

      case WM_DESTROY:  // message: window being destroyed
         PostQuitMessage(0);
         break;

      default:          // Passes it on if unproccessed
         return (DefWindowProc(hWnd, message, wParam, lParam));
   }

   return NULL;
}


//****************************************************************************
// Function: About()
//
// Purpose:  Processes messages for "About" dialog box
//
// Parameters:
//    HWND   hDlg
//    UINT   message
//    WPARAM wParam
//    LPARAM lParam
//
// Returns:
//    BOOL
//
// Comments:
//
// History:    Date        Author         Reason
//
//****************************************************************************

BOOL FAR PASCAL About(HWND   hDlg,      // window handle of the dialog box
                      UINT   message,   // type of message
                      WPARAM wParam,    // message-specific information
                      LPARAM lParam)    // message-specific information
{
   switch (message)
   {
      case WM_INITDIALOG:  // message: initialize dialog box
         return TRUE;

      case WM_COMMAND:     // message: received a command
         if (wParam == IDOK || wParam == IDCANCEL)
         {
            EndDialog(hDlg, TRUE);  // Exits the dialog box
            return TRUE;
         }
         break;
   }

   return FALSE;           // Didn't process a message
}


//****************************************************************************
// Function: ROPopupProc()
//
// Purpose:  Resister's this window class
//
// Parameters:
//    HANDLE hInstance
//
// Returns:
//    BOOL
//
// Comments:
//
// History:    Date        Author         Reason
//             7/7/92      Pat Schreiber  Created
//
//****************************************************************************

BOOL InitROPopup(HANDLE hInstance)    // Handle to this app's instance
{
   WNDCLASS wc;

   // Fill in the window class structure

   wc.style         = NULL;           // Class style(s)
   wc.lpfnWndProc   = ROPopupProc;    // Class window procedure
   wc.cbClsExtra    = 0;              // No per-class extra data
   wc.cbWndExtra    = 0;              // No per-window extra data
   wc.hInstance     = hInstance;      // Application that owns the class
   wc.hIcon         = NULL;           // No class icon
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW); // Use arrow cursor
   wc.hbrBackground = GetStockObject(NULL_BRUSH);  // Use NULL brush
   wc.lpszMenuName  = NULL;           // Name of menu resource in .RC file
   wc.lpszClassName = "ropopup";      // Name used in call to CreateWindow

   // Register the window class

   return RegisterClass(&wc);
}


//****************************************************************************
// Function: ROPopupProc()
//
// Purpose:  Processes messages for read-only popup window
//
// Parameters:
//    HWND   hWnd
//    UINT   message
//    WPARAM wParam
//    LPARAM lParam
//
// Returns:
//    long
//
// Comments:
//
// History:    Date        Author         Reason
//             7/7/92      Pat Schreiber  Created
//
//****************************************************************************

long FAR PASCAL ROPopupProc(HWND   hWnd,    // window handle
                            UINT   message, // type of message
                            WPARAM wParam,  // additional information
                            LPARAM lParam)  // additional information
{
   static HBITMAP hbm=NULL;
   static HBRUSH  hbr=NULL;

   switch (message)
   {
      case WM_CREATE:
         {
            int aPattern[] = {0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55};

            SetCapture(hWnd);
            hbm = CreateBitmap(8, 8, 1, 1, aPattern);
            hbr = CreatePatternBrush(hbm);
         }
         break;

      case WM_PAINT:
         {
            PAINTSTRUCT ps;
            RECT        rc, rect;
            HPEN        hpen, hpenOld;
            HBRUSH      hbrOld;
            char        szText[MAX_LEN];
            int         nLen;

            BeginPaint(hWnd, &ps);

            // Draw border for this popup window

            GetClientRect(hWnd, &rc);

            hpen    = CreatePen(PS_SOLID, 2, GetSysColor(COLOR_WINDOWFRAME));
            hpenOld = SelectObject(ps.hdc, hpen);
            Rectangle(ps.hdc,
                      rc.left + 1,
                      rc.top + 1,
                      rc.right - SD_WIDTH,
                      rc.bottom - SD_HEIGHT);
            SelectObject(ps.hdc, hpenOld);
            DeleteObject(hpen);

            // Write some text to this window

            nLen = LoadString(ghInst, IDS_ROPTEXT, szText, MAX_LEN);

            rect.left   = rc.left + 1 + cxChar;
            rect.top    = rc.top + 1 + cyChar;
            rect.right  = rc.right - SD_WIDTH - cxChar;
            rect.bottom = rc.bottom - SD_HEIGHT - cyChar;

            DrawText(ps.hdc, (LPSTR)szText, nLen, &rect, DT_WORDBREAK);

            // Get our brush for the shadow

            UnrealizeObject(hbr);
            hbrOld = SelectObject(ps.hdc, hbr);

            // bottom shadow

            PatBlt(ps.hdc,
                   rc.left + SD_OFFSET,
                   rc.bottom - SD_HEIGHT,
                   rc.right - SD_OFFSET,
                   SD_HEIGHT,
                   0xA000C9);

            // right-side shadow

            PatBlt(ps.hdc,
                   rc.right - SD_WIDTH,
                   rc.top + SD_OFFSET,
                   SD_WIDTH,
                   rc.bottom,
                   0xA000C9);

            SelectObject(ps.hdc, hbrOld);

            EndPaint(hWnd, &ps);
         }
         break;

      case WM_KEYDOWN:
      case WM_SYSKEYDOWN:
      case WM_LBUTTONDOWN:
      case WM_MBUTTONDOWN:
      case WM_RBUTTONDOWN:
         ReleaseCapture();
         DeleteObject(hbr);
         DeleteObject(hbm);
         DestroyWindow(hWnd);
         break;

      default:          // Passes it on if unproccessed
         return (DefWindowProc(hWnd, message, wParam, lParam));
   }

   return NULL;
}
